@@ -14,7 +14,8 @@
 #'
 #' @examples
 #' subset <- GetWinedata.subset()
-#' subset.spectra = as.matrix(subset$Spectra)
+#' # to reduce the example time we only select spectra 1 & 2
+#' subset.spectra = as.matrix(subset$Spectra)[1:2,] 
 #' subset.ppm = as.numeric(subset$PPM)
 #' 
 #' test.peaks <- getWaveletPeaks(Y.spec=subset.spectra, 
@@ -132,8 +133,13 @@ PeakFilling <- function(Y.grouped, Y.spec, max.index.shift = 10, window.width =
                     
                     
                     res[sapply(res, is.null)] <- NA
-                    if (length(res) != 6) 
-                        res <- rep(NA, 6)  # !important after sapply <- NA! : sometimes the $peakCenterIndex from tuneInPeakInfo returns 'numeric(0)', throw it away
+                    # !important after sapply <- NA! : sometimes the $peakCenterIndex from tuneInPeakInfo returns 'numeric(0)', throw it away
+                    if (length(res) != 6) {
+                        res <- rep(NA, 6)
+                    }
+                    if(length(res) != length(unlist(res))){
+                        res <- rep(NA, 6)
+                    }
                     res <- unlist(res)
                     filled[k, ] <- res
                     
